      FUNCTION ABH2O(T,P,RHO,F)
C
C  NAME- abh2o.f    LANGUAGE- FORTRAN 77
C
C PURPOSE- COMPUTE ABSORPTION COEF IN ATMOSPHERE DUE TO WATER VAPOR
C This version should not be used with a line list older than Dec.31, 2020,
c nor the new list with an older version of this subroutine.
C 
      IMPLICIT NONE
C  CALLING SEQUENCE PARAMETERS-
C    SPECIFICATIONS
      REAL T,P,RHO,F,ABH2O
C      NAME    UNITS    I/O  DESCRIPTON            VALID RANGE
C      T       KELVIN    I   TEMPERATURE
C      P       MILLIBAR  I   PRESSURE              .1 TO 1000
C      RHO     G/M**3    I   WATER VAPOR DENSITY
C      F       GHZ       I   FREQUENCY             
C      ABH2O   NEPERS/KM O   POWER ABSORPTION COEFFICIENT
C
C   Multiply ABH2O by 4.343 to obtain dB/km.
c
c   Line parameters will be read from file h2o_sdlist.asc; intensities should
c   include the isotope abundance factors.
C   This version uses a line-shape cutoff and a speed-dependent shape factor.
C   No T-dependence of SD parameter delta2 in this version.
C
C   REFERENCES FOR EQUATIONS
c    P.W. Rosenkranz, Chap. 2 in ATMOSPHERIC REMOTE SENSING BY MICROWAVE 
c     RADIOMETRY (M.A. Janssen, ed., 1993) http://hdl.handle.net/1721.1/68611
c    P. W. Rosenkranz, Radio Sci., 33(4), 919-928 (1998) doi:10.1029/98RS01182
c    P. W. Rosenkranz and D. Cimini, IEEE Trans. Geosci. Rem. Sens., vol.57(12),
c     9702-9708 (2019) doi:10.1109/TGRS.2019.2928570

C   REVISION HISTORY-
C    DATE- OCT.6, 1988 EQS. AS PUBL. IN 1993 
C     OCT.4, 1995  PWR- USE CLOUGH'S DEFINITION OF LOCAL LINE
C                   CONTRIBUTION,  HITRAN INTENSITIES, ADD 7 LINES.
C     OCT. 24, 95  PWR -ADD 1 LINE.
C     JULY 7, 97   PWR -SEPARATE COEFF. FOR SELF-BROADENING, 
C                       REVISED CONTINUUM.
C     Mar. 2, 2003   PWR - LINE SHIFT
c     Nov. 3, 2012 intensities at base T=296K, get line param. from file.
c     Aug. 6, 2015 read continuum param from the file also. 
c     June 19, 2018 changed file format, separate shift for self & foreign gas
c     Dec. 11, 2018 speed-dependent parameters only for 22 Ghz line
c     Feb. 19, 2019 read sd parameters from the input file.
c     June 25, 2020 SD-delta2 included in the 183 GHz line shape.
c     Jan. 6,  2021 free-form input of parameters, add T-dep of SD.
c     May 20, 2021 detailed print at first call.
C     June 16, 2021 use constants.inc
c     Apr. 6, 2022 - update for Fortran 2018 (no diff in computation)
c     Aug. 22, 2022 added test of freq diff
C
C   LOCAL VARIABLES:
      INTEGER NLINES,I,J,MAXLINES,MOLEC
      PARAMETER (MAXLINES=100)
      REAL DF(2),S1(MAXLINES),B2(MAXLINES),W0(MAXLINES),FL(MAXLINES),
     & W0S(MAXLINES),X(MAXLINES),XS(MAXLINES),SH(MAXLINES),
     & SHS(MAXLINES),XH(MAXLINES),XHS(MAXLINES),
     & AAIR(MAXLINES),ASELF(MAXLINES),W2(MAXLINES),W2S(MAXLINES),
     & D2(MAXLINES),D2S(MAXLINES),XW2(MAXLINES),XW2S(MAXLINES)
      REAL PVAP,PDA,TI,TI2,SUMM,WIDTH0,WIDTH2,WSQ,S,BASE,RES,CON
      REAL SHIFT,SHIFTS,SHIFTF,SAIR,SSELF,TILN,W2air,W2self
      REAL WAIR,WSELF,REFTLINE,REFTCON,CF,CS,XCF,XCS
      REAL DELTA2,D2AIR,D2SELF
      double complex dcerror,Xc,Xrt,pxw,SD
      SAVE NLINES,S1,B2,W0,FL,W0S,X,XS,SH,INIT,REFTLINE,REFTCON,
     & CF,CS,XCF,XCS,SHS,XH,XHS,AAIR,ASELF,W2,W2S,D2,D2S,XW2,XW2S
      LOGICAL INIT,DETAILS
      CHARACTER*4 HEAD
      INCLUDE 'constants.inc'
      DATA INIT/.TRUE./,DETAILS/.FALSE./
C
C   initialization section
      IF(INIT) THEN
        OPEN(80,FILE='h2o_sdlist.asc',
     &   STATUS='OLD',FORM='FORMATTED')
        REFTLINE = 296. !reference T for lines
        READ(80,1) HEAD !header in linedata
1       FORMAT(A4)
        IF(DETAILS)
     &  WRITE(*,*) 'H2O   freq    S(296K)     B  W0air XWair W0slf ',
     &'XWslf  Dair XDair Dself XDslf  Aair Aself W2air XW2air W2slf ',
     &'XW2slf D2air D2slf'  ! header for detailed print
        DO I=1,MAXLINES
C    read line parameters; units: GHz, Hz*cm^2, MHz/mb
          READ(80,*) MOLEC,FL(i),S1(i),B2(i),Wair,X(i),Wself,Xs(i),
     &    Sair,Xh(I),Sself,Xhs(I),Aair(I),Aself(I),W2air,XW2(I),
     &    W2self,XW2S(I),D2air,D2self
          IF(MOLEC.LE.0) GOTO 10 !no more lines, so exit from loop
          NLINES = I
          IF(DETAILS)
     &    WRITE(*,2) MOLEC,FL(i),S1(i),B2(i),Wair,X(i),Wself,Xs(i),
     &    Sair,Xh(I),Sself,Xhs(I),Aair(I),Aself(I),W2air,XW2(I),
     &    W2self,XW2S(I),D2air,D2self !            detailed print
2         FORMAT(I3,F8.3,E12.4,17F6.2)
          W0(I) = Wair/1000. !convert to GHz/mb
          W0S(I) = Wself/1000.
          W2(I) = W2air/1000.
          W2S(I) = W2self/1000.
          SH(I) = Sair/1000.
          SHS(I) = Sself/1000.
          D2(I) = D2air/1000.
          D2S(I) = D2self/1000.
          IF(Xh(I).LE.0.) Xh(I) = X(I)
          IF(Xhs(I).LE.0.) Xhs(I) = Xs(I)
          IF(XW2(I).LE.0.) XW2(I) = X(I)
          IF(XW2S(I).LE.0.) XW2S(I) = Xs(I)
        END DO
        WRITE(*,*) 'WARNING: maximum H2O lines reached'
10      CONTINUE !finished the line list
C    read continuum parameters; units: Kelvin, 1/(km*mb^2*GHz^2)
        READ(80,*) REFTCON,CF,Xcf,CS,Xcs
        CLOSE(80)
        INIT = .FALSE.
        WRITE(*,*) NLINES,' H2O lines used' !summary print
        IF(DETAILS) WRITE(*,3)  REFTCON,CF,Xcf,CS,Xcs !  detailed print
3       FORMAT('H2O continuum @',F5.0,E16.3,F4.1,E10.3,F4.1)
      ENDIF
C
      IF(RHO.LE.0.) THEN
        ABH2O = 0.
        RETURN
      ENDIF
      PVAP = RH2O*RHO*T
      PDA = P -PVAP  
C
C   CONTINUUM TERMS
      TI = REFTCON/T
c   Xcf and Xcs include 3 for density & stimulated emission
      CON = (CF*PDA*TI**XCF + CS*PVAP*TI**XCS)*PVAP*F*F
C
C   ADD RESONANCES
      TI = REFTLINE/T
      TILN = ALOG(TI)
      TI2 = EXP(2.5*TILN) ! = TI**2.5
      SUMM = 0.
      DO 30 I=1,NLINES
      IF(ABS(F-FL(I)).GT.750.1) GOTO 30 !skip if F is outside the truncated l.s.
      WIDTH0 = W0(I)*PDA*TI**X(I) + W0S(I)*PVAP*TI**XS(I)
      IF(W2(I).GT.0.) THEN
       WIDTH2 = W2(I)*PDA*TI**XW2(I) + W2S(I)*PVAP*TI**XW2S(I)
      ELSE
       WIDTH2 = 0.
      ENDIF
      DELTA2 = D2(I)*PDA + D2S(I)*PVAP ! DELTA2 assumed independent of T 
      SHIFTF = SH(I)*PDA*(1.-AAIR(I)*TILN)*TI**XH(I)
C Frost model mod. by Smith et al, Spectrochimica Acta v.48A(9), 1257-72 (1992)
      SHIFTS = SHS(I)*PVAP*(1.-ASELF(I)*TILN)*TI**XHS(I)
      SHIFT = SHIFTF + SHIFTS
      WSQ = WIDTH0**2
c  line intensities include isotopic abundance
      S = S1(I)*TI2*EXP(B2(I)*(1.-TI)) 
      DF(1) = F - FL(I) - SHIFT
      DF(2) = F + FL(I) + SHIFT
C  USE CLOUGH'S DEFINITION OF LOCAL LINE CONTRIBUTION
      BASE = WIDTH0/(562500. + WSQ)

C  DO FOR POSITIVE AND NEGATIVE RESONANCES
      RES = 0.
      DO  J=1,2 ! sum 2 resonances for each line
       IF(WIDTH2.GT.0. .AND. J.EQ.1 .AND. ABS(DF(J)).LT.10.*WIDTH0) THEN
C--------speed-dependent resonant shape factor, minus base
         Xc = cmplx((WIDTH0-1.5*WIDTH2),DF(J)+1.5*DELTA2)/
     &    cmplx(WIDTH2,-DELTA2)
         Xrt = cdsqrt(Xc)
         pxw = 1.77245385090551603d0*Xrt*dcerror(-dimag(Xrt),dble(Xrt))
         SD = 2.d0*(1.d0-pxw)/cmplx(WIDTH2,-DELTA2)
         RES = RES + REAL(SD) - BASE
C--------end of speed-dependence section
       ELSEIF(ABS(DF(J)).LT.750.) THEN
         RES = RES + WIDTH0/(DF(J)**2+WSQ) - BASE !Lorentzian - base
       ENDIF
      END DO
      SUMM = SUMM + S*RES*(F/FL(I))**2 !sum of shape factors over all lines
30    CONTINUE
      ABH2O = 1.E-10*RHO*SUMM/(PI*H2OM) + CON
      RETURN
      END
